# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='product',
            name='value',
            field=models.DecimalField(default=0, max_digits=11, decimal_places=2),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import models
from products.models import Product

# Create your models here.

PAYMENT_TYPE = (
		(1, u"boleto"),
		(2, u"cartão de crédito"),
		(3, u"débito"),
	)

GENDER = (
	(0, u"Male"),
	(1, u"Female")
)


class Order(models.Model):
	data = models.DateField(auto_now_add=True, verbose_name='Data do Cadastro')
	payment_type = models.PositiveSmallIntegerField(default=None, choices=PAYMENT_TYPE, blank=True, null=True)
	email = models.EmailField(null=True)
	delivered = models.BooleanField(default=False)
	total_value = models.DecimalField(max_digits=11, decimal_places=2)
	gender = models.NullBooleanField(choices=GENDER, null=True, blank=True, default=None)

	def __unicode__(self):
		return str(self.id)


class OrderItem(models.Model):
	class Meta:
		verbose_name, verbose_name_plural = u"Order Iten", u"Order Itens"

	order = models.ForeignKey(Order)
	product = models.ForeignKey(Product)
	quantity = models.PositiveSmallIntegerField(verbose_name=u'quantity sold')
	value = models.DecimalField(max_digits=11, decimal_places=2)
	total = models.DecimalField(max_digits=11, decimal_places=2, blank=True, default=0)

	def save(self, *args, **kwargs):
		self.total = self.quantity * self.value
		super(OrderItem, self).save(*args, **kwargs)


class OrderProxy(Order):
	class Meta:
		verbose_name, verbose_name_plural = u"Report Order", u"Report Orders"
		proxy = True


class ProductProxy(Product):
	class Meta:
		verbose_name, verbose_name_plural = u"Report Order Item", u"Report Order Items"
		proxy = True



"""
Django settings for test_django_admin_report project.

For more information on this file, see
https://docs.djangoproject.com/en/1.6/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.6/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.6/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '1shaum3ab9sbmfd4*&7re_k3jpu%xqa#r_h1mqct2tx7@=8zh)'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

TEMPLATE_DEBUG = DEBUG

ALLOWED_HOSTS = ['*']

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.contrib.auth.context_processors.auth",
    'django.core.context_processors.debug',
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.static",
    "django.core.context_processors.tz",
    "django.contrib.messages.context_processors.messages",
    'django.core.context_processors.request',
)

# Application definition

INSTALLED_APPS = (
    'admin_report',
    'suit',
    'import_export',
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'orders',
    'products',
    'django_extensions',
    'debug_toolbar',
)

DEBUG_TOOLBAR_PATCH_SETTINGS = False
INTERNAL_IPS = "127.0.0.1"

MIDDLEWARE_CLASSES = (
    'debug_toolbar.middleware.DebugToolbarMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

ROOT_URLCONF = 'test_django_admin_report.urls'

WSGI_APPLICATION = 'test_django_admin_report.wsgi.application'


# Database
# https://docs.djangoproject.com/en/1.6/ref/settings/#databases

DATABASES = {
    'default': {
                'ENGINE': 'django.db.backends.mysql',
                'NAME': 'django_chart_report',
                'USER': 'root',
                'PASSWORD': '123456',
                'HOST': '',
                'PORT': '',
        }
}

# Internationalization
# https://docs.djangoproject.com/en/1.6/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.6/howto/static-files/

STATIC_URL = '/static/'

TEMPLATE_DIRS = (
    os.path.join(BASE_DIR, 'templates'),
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0002_product_value'),
    ]

    operations = [
        migrations.CreateModel(
            name='Order',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('data', models.DateField(auto_now_add=True, verbose_name=b'Data do Cadastro')),
                ('payment_type', models.PositiveSmallIntegerField(default=None, null=True, blank=True, choices=[(1, 'boleto'), (2, 'cart\xe3o de cr\xe9dito'), (3, 'd\xe9bito')])),
                ('email', models.EmailField(max_length=75, null=True)),
                ('delivered', models.BooleanField(default=False)),
                ('total_value', models.DecimalField(max_digits=11, decimal_places=2)),
                ('gender', models.NullBooleanField(default=None, choices=[(0, 'Male'), (1, 'Female')])),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='OrderItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('quantity', models.PositiveSmallIntegerField(verbose_name='quantity sold')),
                ('value', models.DecimalField(max_digits=11, decimal_places=2)),
                ('order', models.ForeignKey(to='orders.Order')),
                ('product', models.ForeignKey(to='products.Product')),
            ],
            options={
                'verbose_name': 'Order Iten',
                'verbose_name_plural': 'Order Itens',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='OrderProxy',
            fields=[
            ],
            options={
                'verbose_name': 'Order',
                'proxy': True,
                'verbose_name_plural': 'Orders',
            },
            bases=('orders.order',),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('products', '0002_product_value'),
        ('orders', '0003_auto_20141225_2344'),
    ]

    operations = [
        migrations.CreateModel(
            name='OrderItemProxy',
            fields=[
            ],
            options={
                'verbose_name': 'Order Item',
                'proxy': True,
                'verbose_name_plural': 'Order Items',
            },
            bases=('products.product',),
        ),
    ]



from django.conf.urls import patterns, include, url
from django.conf import settings
from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    # Examples:
    # url(r'^$', 'test_django_admin_report.views.home', name='home'),
    # url(r'^blog/', include('blog.urls')),

    url(r'^admin/', include(admin.site.urls)),
)

if settings.DEBUG:
    import debug_toolbar
    urlpatterns += patterns('',
        url(r'^__debug__/', include(debug_toolbar.urls)),
    )


from django.db import models


class Product(models.Model):
	name = models.CharField("Product name", max_length=255)
	value = models.DecimalField(max_digits=11, decimal_places=2)

	def __unicode__(self):
		return self.name



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('orders', '0002_orderitem_total'),
    ]

    operations = [
        migrations.AlterField(
            model_name='orderitem',
            name='total',
            field=models.DecimalField(default=0, max_digits=11, decimal_places=2, blank=True),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-

from django.contrib import admin
from .models import *
from django.db.models import Sum, Avg, Count, Min, Max
from admin_report.mixins import ChartReportAdmin
from import_export import resources
from import_export.admin import ExportMixin
from import_export import fields
from django.utils import formats


class AdminNoAddPermissionMixin(object):
    def has_add_permission(self, request):
        return False


class OrderItemInline(admin.TabularInline):
    model = OrderItem
    extra = 1


class OrderAdmin( admin.ModelAdmin ):
	# search_fields = ('nome', 'email', 'cpf', 'rg')
	list_display = ('data', 'payment_type', 'email', 'delivered', 'total_value', 'gender')
	list_filter = ('data', 'payment_type')
	inlines = [OrderItemInline]
	# date_hierarchy = 'data'
	# exclude = ('grupo_tributacao',)
admin.site.register( Order, OrderAdmin )


class ReportOrderAdmin(ExportMixin, AdminNoAddPermissionMixin, ChartReportAdmin):
	list_filter = ['payment_type', 'email', 'delivered']
	list_display = ('data', 'payment_type', 'email', 'delivered', 'total_value', 'gender')

	# group_by = "gender"

	report_aggregates = (
		('total_value', Sum, "<b>Total vendido</b>"),
		('total_value', Avg, "<b>Valor médio</b>"),
		('total_value', Count, "<b>Número de pedidos</b>"),
	)

admin.site.register( OrderProxy, ReportOrderAdmin )


class ReportOrderItemsResource(resources.ModelResource):
    orderitem__value__avg = fields.Field(attribute="orderitem__value__avg", column_name="valor médio do produto")
    orderitem__value__max = fields.Field(attribute="orderitem__value__max", column_name="maior valor de venda")
    orderitem__value__min = fields.Field(attribute="orderitem__value__min", column_name="menor valor de venda")
    orderitem__quantity__sum = fields.Field(attribute="orderitem__quantity__sum", column_name="total de itens vendidos")
    orderitem__total__sum = fields.Field(attribute="orderitem__total__sum", column_name="valor total vendido")

    class Meta:
        model = ProductProxy
        fields = ('name', 'orderitem__value__avg', 'orderitem__value__max', 'orderitem__value__min', 'orderitem__quantity__sum', 'orderitem__total__sum',)
        export_order = fields

    def dehydrate_orderitem__value__avg(self, obj):
        return u"R$ {0}".format(formats.number_format(obj.orderitem__value__avg, 2))

    def dehydrate_orderitem__value__max(self, obj):
        return formats.number_format(obj.orderitem__value__max, 2)

    def dehydrate_orderitem__value__min(self, obj):
        return formats.number_format(obj.orderitem__value__min, 2)

    def dehydrate_orderitem__quantity__sum(self, obj):
        return formats.number_format(obj.orderitem__quantity__sum, 2)

    def dehydrate_orderitem__total__sum(self, obj):
        return formats.number_format(obj.orderitem__total__sum, 2)


class ReportOrderItemsAdmin(ExportMixin, AdminNoAddPermissionMixin, ChartReportAdmin):
	resource_class = ReportOrderItemsResource
	list_filter = ['name', 'orderitem__order__payment_type', ]
	list_display = ('name', 'valor_atual', 'orderitem__value__avg', 'orderitem__value__max', 'orderitem__value__min', 'orderitem__quantity__sum', 'orderitem__total__sum',)

	def valor_atual(self, obj):
		return obj.value
	valor_atual.short_description = 'valor atual do produto'
	valor_atual.admin_order_field = 'value'
    # valor_atual.allow_tags = True

	report_annotates = (
		("orderitem__quantity", Sum, "total de itens vendidos"),
		("orderitem__total", Sum, "valor total vendido"),
		# ("orderitem__total", Count, "count total vendido"),
		("orderitem__value", Avg, "valor médio do produto"),
		("orderitem__value", Max, "maior valor de venda"),
		("orderitem__value", Min, "menor valor de venda"),
		# ("orderitem__valor_sem_desconto", Min, "menor valor do produto"),
	)

	report_aggregates = (
		('orderitem__total__sum', Sum, "<b>Total: R$ %value</b>"),
		('orderitem__quantity', Sum, "total de itens vendidos"),
	)

admin.site.register( ProductProxy, ReportOrderItemsAdmin )



from distutils.core import setup
from setuptools import find_packages

VERSION = __import__("admin_report").__version__

CLASSIFIERS = [
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Topic :: Software Development',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 3',
]

install_requires = [
    'django-daterange-filter',
]

setup(
    name="django-admin-report",
    packages=find_packages(exclude=["tests"]),
    description="Django application and library for create reports using all power of the ORM Django",
    version=VERSION,
    author="Mateus Vanzo de Padua",
    author_email="mateuspaduaweb@gmail.com",
    license='MIT License',
    platforms=['OS Independent'],
    url="https://github.com/mateuspadua/django-admin-report",
    keywords=['report', 'django', 'admin', 'group_by', 'annotate', 'agregate', 'ORM'],
    include_package_data=True,
    install_requires=install_requires,
    classifiers=CLASSIFIERS,
)



from django.contrib import admin
from products.models import Product


class ProductAdmin( admin.ModelAdmin ):
	# search_fields = ('nome', 'email', 'cpf', 'rg')
	list_display = ('name', 'value')
	# date_hierarchy = 'data'
	# exclude = ('grupo_tributacao',)
admin.site.register( Product, ProductAdmin )



# -*- coding: utf-8 -*-
from django.contrib import admin
from django.contrib.admin.views.main import ChangeList
from django.db.models.query import ValuesQuerySet
from django.db.models import Sum, Avg, Count, Max, Min
from django.contrib.admin.validation import ModelAdminValidator
from django.http import Http404, HttpResponseRedirect
from django.contrib.admin.options import IncorrectLookupParameters
from django.core.exceptions import SuspiciousOperation, ImproperlyConfigured
from daterange_filter.filter import DateRangeFilter
from django.db.models import ForeignKey
from django.db.models.constants import LOOKUP_SEP
from django.db.models.sql.constants import QUERY_TERMS
import copy
import types
import uuid
import sys
from django.utils import formats

map_aggregates = ((Sum, "__sum"), (Count, "__count"), (Avg, "__avg"), (Max, "__max"), (Min, "__min"))


def function_builder(name_function, attr, title_column):
    def new_function(self, obj):
        return getattr(obj, attr)
    new_function.__name__ = name_function
    if title_column:
        new_function.short_description = title_column
    new_function.admin_order_field = attr
    new_function.allow_tags = True
    return new_function


class ChangeListChartReport(ChangeList):

    result_aggregate = []

    def get_queryset(self, request):

        # qs = super(self.__class__, self).get_queryset(request)
        # First, we collect all the declared list filters.
        (self.filter_specs, self.has_filters, remaining_lookup_params,
            filters_use_distinct) = self.get_filters(request)

        # Then, we let every list filter modify the queryset to its liking.

        qs = self.root_queryset
        # print "#### qs antes ####"
        # print qs.query
        all_query = {}
        for filter_spec in self.filter_specs:
            # print "---- filter_spec ----"
            # print filter_spec.used_parameters
            if filter_spec.used_parameters and filter_spec.used_parameters.values()[0]:
                if isinstance(filter_spec, DateRangeFilter):
                    if filter_spec.form.is_valid():
                        # get no null params
                        filter_params = dict(filter(lambda x: bool(x[1]),
                                                    filter_spec.form.cleaned_data.items()))
                        all_query.update(**filter_params)
                else:
                    all_query.update(filter_spec.used_parameters)

            # comentando a versao do Django, pois da maneira abaixo ele intercala os join gerando joins extras, o que resulta em um resultado errado, nas agregaçoes SUM, COUNT, etc...
            # new_qs = filter_spec.queryset(request, qs)
            # if new_qs is not None:
            #     qs = new_qs

        # coloca o dicionário de uma vez só, assim dessa forma não há problema de criar joins "desnessárias" para filtros que apontam para as mesma tabelas estrangeiras
        # como explicado logo acima
        qs = qs.filter(**all_query)

        # print "#### qs depois ####"
        # print qs.query

        try:
            # Finally, we apply the remaining lookup parameters from the query
            # string (i.e. those that haven't already been processed by the
            # filters).
            qs = qs.filter(**remaining_lookup_params)
        except (SuspiciousOperation, ImproperlyConfigured):
            # Allow certain types of errors to be re-raised as-is so that the
            # caller can treat them in a special way.
            raise
        except Exception as e:
            # Every other error is caught with a naked except, because we don't
            # have any other way of validating lookup parameters. They might be
            # invalid if the keyword arguments are incorrect, or if the values
            # are not in the correct type, so we might get FieldError,
            # ValueError, ValidationError, or ?.
            raise IncorrectLookupParameters(e)

        if not qs.query.select_related:
            qs = self.apply_select_related(qs)

        # Set ordering.
        ordering = self.get_ordering(request, qs)
        qs = qs.order_by(*ordering)

        # Apply search results
        qs, search_use_distinct = self.model_admin.get_search_results(
            request, qs, self.query)

        # Remove duplicates from results, if necessary
        if filters_use_distinct | search_use_distinct:
            qs = qs.distinct()

        if self.model_admin.annotate_fields:
            # qs = self.root_queryset

            # all_used_parameters = {}
            # qs = self.root_queryset
            # (self.filter_specs, self.has_filters, remaining_lookup_params, filters_use_distinct) = self.get_filters(request)
            # for filter_spec in self.filter_specs:
            #     # props = ['choices', 'create', 'expected_parameters', 'field', 'field_path', 'has_output', 'lookup_kwarg', 'lookup_val', 'queryset', 'register', 'template', 'title', 'used_parameters']
            #     # for p in props:
            #     #    print "{0}: {1}".format(p, getattr(filter_spec, p))
            #     # print "#####################################################"
            #     new_qs = filter_spec.queryset(request, qs)
            #     if new_qs is not None:
            #         qs = new_qs
            #     all_used_parameters.update(filter_spec.used_parameters)

            # if all_used_parameters:
            #     qs = qs.filter(**all_used_parameters)

            # estudar essa parte, esta estranho
            if self.model_admin.group_by:
                qs = qs.values(*self.model_admin.group_by)

            # faz uma copia antes de chamar o metodo annotate, pois para campos aggregate que não
            # representam um campo annotate, não se pode ter o annotate na query
            self.query_to_normal_aggregate = qs

            qs = qs.annotate(*self.model_admin.annotate_fields_2, **self.model_admin.annotate_fields)

            # Set ordering.
            ordering = self.get_ordering(request, qs)
            qs = qs.order_by(*ordering)

            # Apply search results
            qs, search_use_distinct = self.model_admin.get_search_results(request, qs, self.query)

            # print qs.query
            # print "get_queryset"
        else:
            self.query_to_normal_aggregate = qs

        # print "#### query final ####"
        # print qs.query

        return qs

    def get_results(self, request):
        super(self.__class__, self).get_results(request)

        # print self.result_list[0].pedidoitem__valor_unitario_sem_descontos__avg
        # print self.result_list[0].pedidoitem__valor_unitario_sem_descontos__max

        # print "get_results"
        # print self.queryset.query

        if isinstance(self.result_list, ValuesQuerySet):
            pk_name = self.model._meta.pk.name
            try:
                ids = [i[pk_name] for i in self.result_list]
            except KeyError:
                ids = []

            result_list_in_model = self.model.objects.filter(pk__in=ids)
            result_list_in_model_dict = {}
            for item_model in result_list_in_model:
                result_list_in_model_dict[getattr(item_model, pk_name)] = item_model

            # print "result_list_in_model_dict ========="
            # print result_list_in_model_dict

            new_result_list = []
            for row in self.result_list:
                if pk_name in row and result_list_in_model_dict:
                    new_row = copy.deepcopy(result_list_in_model_dict[row[pk_name]])
                else:
                    new_row = self.model()

                for key, value in list(row.items()):
                    if hasattr(new_row, key):
                        field_object, model, direct, m2m = self.model._meta.get_field_by_name(key)
                        if (m2m or isinstance(field_object, ForeignKey)) is True:
                            continue

                    setattr(new_row, key, value)

                new_result_list.append(new_row)

            self.result_list = new_result_list

        self.get_result_aggregate()

    def get_result_aggregate(self):
        # print self.list_display

        result_aggregate = []
        result_aggregate_queryset = None
        result_aggregate_from_normal_queryset = {}
        result_aggregate_from_annotate_queryset = {}
        # from django.db import connection
        # connection.queries = []
        qs = self.queryset
        if self.model_admin.aggregate_fields_from_normal:
            # print "normal"
            result_aggregate_from_normal_queryset = self.query_to_normal_aggregate.aggregate(*self.model_admin.aggregate_fields_from_normal)
            # print "########## result_aggregate_from_normal_queryset ##########"
            # print result_aggregate_from_normal_queryset

        if self.model_admin.aggregate_fields_from_annotate:
            # print "annotate"
            result_aggregate_from_annotate_queryset = qs.aggregate(*self.model_admin.aggregate_fields_from_annotate)
            # print "########## result_aggregate_from_annotate_queryset ##########"
            # print result_aggregate_from_annotate_queryset

        # print "#######"
        # print qs.query
        # print connection.queries

        result_aggregate_queryset = dict(result_aggregate_from_normal_queryset, **result_aggregate_from_annotate_queryset)

        # print result_aggregate_queryset

        if result_aggregate_queryset:
            for column in self.list_display:
                result_aggregate_temp = None
                if column in self.model_admin.map_display_fields_and_aggregate:
                    for aggregate in self.model_admin.map_display_fields_and_aggregate[column]:
                        # clean_name_field = aggregate[0][:-len(aggregate[0][aggregate[0].rfind("__"):])]
                        pos_value_place_holder = aggregate[2].find("%value")

                        aggregate_string_replace = "<strong>{0}:</strong> {1}"
                        if pos_value_place_holder != -1:
                            aggregate_string_replace = aggregate[2].replace("%value", "{1}")

                        if isinstance(result_aggregate_queryset[aggregate[0]], float):
                            label_foot = formats.number_format(result_aggregate_queryset[aggregate[0]], 2)
                        else:
                            label_foot = formats.localize(result_aggregate_queryset[aggregate[0]], use_l10n=True)

                        label_foot = aggregate_string_replace.format(aggregate[2], label_foot)
                        result_aggregate_temp = "{0}<br>{1}".format(result_aggregate_temp, label_foot) if result_aggregate_temp else label_foot
                    # print "@@@@@@@@@@@@@@@@@@@@@@@"
                if result_aggregate_temp:
                    result_aggregate.append(result_aggregate_temp)
                else:
                    result_aggregate.append("")

        # print "################### result_aggregate #####################"
        # print result_aggregate
        self.result_aggregate = result_aggregate


class ModelAdminValidator2(ModelAdminValidator):
    def validate_list_display(self, cls, model):
        # super(ModelAdminValidator2, self).validate_list_display(cls, model)
        pass


class AdminExceptionFieldsFilterMixin(admin.ModelAdmin):

    exception_fields_filter = ()

    def lookup_allowed(self, lookup, value):
        ret = super(AdminExceptionFieldsFilterMixin, self).lookup_allowed(lookup, value)
        if not ret and self.exception_fields_filter:
            parts = lookup.split(LOOKUP_SEP)

            if len(parts) > 1 and parts[-1] in QUERY_TERMS:
                parts.pop()
                clean_lookup = LOOKUP_SEP.join(parts)
                if clean_lookup in self.exception_fields_filter:
                    return True
        return ret


class ChartReportAdmin(admin.ModelAdmin):
    # change_list_template = "admin/relatorios/change_list.html"

    report_annotates = ()
    report_aggregates = ()
    group_by = ()
    validator_class = ModelAdminValidator2

    # def __new__(cls, *args, **kwargs):
        # return super(ChartReportAdmin, cls).__new__(cls, *args, **kwargs)

    class Media:
        css = {
            'all': ('admin/css/django_admin_report/django_admin_report.css',)
        }

    def __init__(self, model, admin_site):
        # print "########################### report_annotates ###############################"
        # print self.report_annotates
        self.annotate_fields = {}
        self.annotate_fields_2 = []
        # self.aggregate_fields = []
        self.aggregate_fields_from_normal = []
        self.aggregate_fields_from_annotate = []
        self.map_annotate_fields = {}
        self.map_display_fields_and_aggregate = {}

        for annotate in self.report_annotates:
            for func, end_field_name in map_aggregates:
                if func == annotate[1]:
                    name_field_annotate = "{0}{1}".format(annotate[0], end_field_name)
                    new_name_field_annotate = "{0}_{1}".format(annotate[0].replace("__", "_"), str(uuid.uuid4()).replace("-", "")[:10])
                    self.map_annotate_fields.update({name_field_annotate: new_name_field_annotate})
                    self.annotate_fields.update({new_name_field_annotate: annotate[1](annotate[0])})
                    self.annotate_fields_2.append(annotate[1](annotate[0]))
                    self.addMethod(function_builder(name_field_annotate, new_name_field_annotate, annotate[2] if len(annotate) == 3 else None))
                    break

        for aggregate in self.report_aggregates:
            for func, end_field_name in map_aggregates:
                if func == aggregate[1]:
                    copy_aggregate = list(aggregate[:])
                    if aggregate[0] in self.map_annotate_fields:
                        new_name_field = self.map_annotate_fields[aggregate[0]]
                    else:
                        new_name_field = aggregate[0]
                    name_field_aggregate = "{0}{1}".format(new_name_field, end_field_name)

                    copy_aggregate[0] = name_field_aggregate

                    new_label = "{0}{1}".format(aggregate[0], end_field_name)
                    if len(copy_aggregate) == 2:
                        copy_aggregate.append(new_label)
                    elif not copy_aggregate[2]:
                        copy_aggregate[2] = new_label

                    if len(copy_aggregate) == 4:  # significa que foi especificado em qual coluna deve aparecer o valor agregado
                        column_display_list = aggregate[3]
                    else:
                        column_display_list = aggregate[0]
                        if column_display_list not in self.list_display:
                            column_display_list = new_label

                    if column_display_list not in self.map_display_fields_and_aggregate:
                        self.map_display_fields_and_aggregate[column_display_list] = []

                    self.map_display_fields_and_aggregate[column_display_list].append(copy_aggregate)
                    break

            if aggregate[0] in self.map_annotate_fields:
                new_name_field = self.map_annotate_fields[aggregate[0]]
                self.aggregate_fields_from_annotate.append(aggregate[1](new_name_field))
            else:
                self.aggregate_fields_from_normal.append(aggregate[1](aggregate[0]))

        # print self.aggregate_fields_from_normal
        # print self.aggregate_fields_from_annotate
        # print "########################## self.map_display_fields_and_aggregate #########################"
        # print self.map_display_fields_and_aggregate

        # ChartReportAdmin.addMethod(function_builder("total_value__sum"))

        self.list_max_show_all = sys.maxsize

        super(ChartReportAdmin, self).__init__(model, admin_site)
        self.list_display_links = (None, )

    # TODO: refazer esse metodo, nao esta legal
    def add_view(self, request, form_url='', extra_context=None):
        return HttpResponseRedirect(request.META["HTTP_REFERER"])  # volta sempre para a mesma pagina

    def addMethod(self, func):
        return setattr(self, func.__name__, types.MethodType(func, self))

    def get_changelist(self, request, **kwargs):
        return ChangeListChartReport